

--- 
--- package para definicao de uma ROM - contantes
---
library IEEE;
use IEEE.std_logic_1164.all;

package ROM is		-- definio de uma rom 4x13
	constant largura : integer := 4;
	subtype palavra is std_logic_vector(1 to largura);
	subtype tamanho is integer range 0 to 12;
	type mem_rom is array (0 to 12) of palavra;
	constant ROM1 : mem_rom := mem_rom'(palavra'("1100"),
	           palavra'("1100"),palavra'("0100"),palavra'("0000"),
	           palavra'("0110"),palavra'("0101"),palavra'("0111"),
	           palavra'("1100"),palavra'("0100"),palavra'("0000"),
	           palavra'("0110"),palavra'("0101"),palavra'("0111") );
end ROM;


--- 
--- contador utilizando as constantes da ROM
---
library IEEE;
use IEEE.std_logic_1164.all;
use work.ROM.all;

entity contador is
		port( clock, reset : in std_logic;   waves : out palavra);
end;

architecture A of contador is
		signal step : tamanho := 0;
begin
		waves <= ROM1(step);	-- contedo da ROM na sada
		process
			begin
			wait until clock'event and clock='1';
			if reset='1' then
				step <= 0;				-- primeiro estado
			elsif step = tamanho'high then
				step <= tamanho'high;		-- tranca !
			else
				step <= step + 1;			-- avana 1 passo
			end if;
		end process;
end A;



---------------------------------------------------------------
-----  test_bench
---------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use work.ROM.all;

entity tb is
end tb;

architecture teste of tb is
	
	component contador
	    port( clock, reset : in std_logic;   waves : out palavra);
	end component;
	
	signal ck,reset : std_logic;
	signal saida : palavra;
	
	begin
	
	   cont1:  contador port map(clock=>ck, reset=>reset, waves=>saida);
	   
	   reset <= '1', '0' after 15ns ;
	   
	   -- gerador de clock
	   process
	    begin
	     ck <= '1' after 10ns, '0' after 20ns;
	     wait for 20ns;
	   end process;
	    
	end teste;


